#####################################################################################
## what...convenience function to determine if the GO id <goclass> belongs to a    ##
## certain <flavor> of the GO hierarchy                                            ##
## goclass...the GO id one is interested in (e.g."GO:0006958")                     ##
## flavor...one of "CC", "MF", "BP"                                                ##
## when...2005-03-09                                                               ##
## who...florian. sohler@ifi.lmu.de                                                ##
#####################################################################################
isFlavor <- function(goclass, flavor) {
  gt <- GOTERM[[goclass]]
  if (is.null(gt))
    return (FALSE)
  return (any(attr(gt, "Ontology")==flavor))
}

#####################################################################################
## what...convenience function to compute a submap of a GO map. Selects all items  ##
## whose names are GO ids that belong to the <flavor> part of the GO hierarchy     ##
## flavor...one of "CC", "MF", "BP"                                                ##
## when...2005-03-09                                                               ##
## who...florian. sohler@ifi.lmu.de                                                ##
#####################################################################################
getFlavorSubmap <- function(map, flavor=c("CC", "MF", "BP")) {
  newmap <- list()
  for (i in 1:length(names(map))) {
    if (isFlavor(names(map)[i], flavor)) {
        newmap <- c(newmap, map[i])
      }
  }
  return(newmap)
}

getGOTerm <- function(goclass)
  unlist(lapply(mget(goclass, GOTERM), attr, "Term"))

makeMap <- function(chip, probes, ontology="BP", upperLimit=300,
                    lowerLimit=5) {
  map <- NULL
  if(chip != "plain") {
    require(chip, character.only = TRUE) ||
    stop("annotation package", chip, "not found")
    fullMap <- as.list(get(paste(chip, "GO2ALLPROBES", sep="")))
  }
  else {
    require("GO", character.only = TRUE) ||
    stop("GO package not found")
    fullMap <- as.list(GOALLLOCUSID)
  }
  i <- 1
  j <- 1
  while(i <= length(fullMap)){
    probeClass <- unique(fullMap[[i]])
    probeClass <- probeClass[probeClass %in% probes]
    indices <- match(probeClass, probes)
    if(length(probeClass) <= upperLimit &&
       length(probeClass) >= lowerLimit) {
      map[[j]] <- indices
      names(map)[[j]] <- names(fullMap)[i]
      j <- j + 1
    }
    i <- i + 1
  }
  map <- getFlavorSubmap(map, ontology)

  ## remove all parents that are identical to any of their children
  changed <- TRUE
  while(changed){
    changed <- FALSE
    for(i in 1:length(map)) {
      term <- names(map)[i]
      if(!exists(term, GOBPCHILDREN))
        next
      children <- get(term, GOBPCHILDREN)
      children <- children[children %in% names(map)]
      redundantChildren <- unlist(lapply(map[children],
                                         function(x) {
                                                   all(map[[term]] %in% x)
                                         }))
      if(any(redundantChildren)) {
        map[term] <- NULL
        changed <- TRUE
        cat("removed", term, "in favor of", names(redundantChildren)[redundantChildren],"\n")
      }
    }
  }
  map
}
