lazyModelBootstrap <- function(models, scores, K=2000, null=1,
                               alt=2, correction="BY", verbose=TRUE,...) {
  ## K is only a suggestion, the true amount of evaluated bootstrap replicons
  ## will be sowhat larger

  correction <- match.arg(correction, p.adjust.methods)

  d2 <- function(x) {
    if(length(x) == 1)
      return(as.matrix(x))
    else
      return(diag(x))
  }

  scoretype <- attr(scores, "scoretype")
  models <- models@pcaModels
  n <- models[[1]]@nobs
  
  
  ## How many different k are there in models
  KS <- unlist(lapply(models, function(x) return(x@k)))
  ks <- table(KS)

  ## Compute distributions of variances
  varDist <- list()
  for(i in 1:length(ks)) {
    k <- as.integer(names(ks))[i]
    filter <- unlist(lapply(models,
                            function(x) return(x@k == k)))
    varDist[[i]] <- matrix(apply(rbind(models[filter]),2,
                                 function(x) return(apply(x[[1]]@scores, 2, var))),
                           nrow=k)
  }

  ## compute bootstrap distributions
  ## n = number of observations
  ## alt = number of components
  ## Sigma = variances of components
  ## rep = number of replicates
  h0dists <- mapply(function(n, alt, Sigma, rep)
                    {
                      replicate(rep,
                                mvrnorm(n, rep(0, alt),
                                        d2(Sigma[,sample(1:ncol(Sigma), 1)])),
                                simplify=FALSE)
                    }, n, as.integer(names(ks)), varDist, K, SIMPLIFY=FALSE)

  ## Compute scores distributions of bootstrap observations
  if(verbose)
    cat("Evaluating scores using", length(h0dists), "*", K, "bootstrap samples\n")

  suppressWarnings(bootClust <- lapply(h0dists,
                                       function(x) {
                                         unlist(mapply(function(y, i, to)
                                                       {
                                                         if(verbose)
                                                           progBar(i, to)
                                                         return(clusterGeneClass(y, scoretype=scoretype)$score)
                                                       }, x, 1:length(x), length(x)))
                                       }))
  
  names(bootClust) <- names(ks)
  
  ## Compute the estimated p-values for observed scores
  pvals <- mapply(function(ratio, termId)
                  {
                    k <- as.character(KS[termId])
                    return((sum(bootClust[[k]] >= ratio)) /
                           length(bootClust[[k]]))
                  }, scores@scores, names(scores@scores))

  names(pvals) <- names(scores@scores)

  res <- NULL
  res$corr <- p.adjust(pvals, correction)
  res$orig <- pvals
  if(verbose)
    cat("Done\n")

  res
}


