.clusterGeneClass <- function(x, clNum=2,
                              scoretype=c("mclust", "weightedsilhouette", "silhouette"),...) {
  
  scoretype <- match.arg(scoretype)

  switch(scoretype,
         silhouette = {
           cl <- list()
           pamcl <- pam(x, clNum)
           sizes <- as.vector(table(pamcl$clustering))
           cl$clustering <- pamcl$clustering
           names(cl$clustering) <- names(x[,1])
           cl$certainty <- rep(1, nrow(x))
           names(cl$certainty) <- names(x[,1])
           v <- summary(silhouette(pamcl))$avg.width

         },
         weightedsilhouette = {
           cl <- list()
           pamcl <- pam(x, clNum)
           sizes <- as.vector(table(pamcl$clustering))
           cl$clustering <- pamcl$clustering
           names(cl$clustering) <- names(x[,1])
           cl$certainty <- rep(1, nrow(x))
           names(cl$certainty) <- names(x[,1])
           v <- .silScore(pamcl)
         },
         mclust = {
           cl <- list()
           ms <- .mclustScore(x, clNum)
           sizes <- as.vector(table(ms$clustering))
           cl$clustering <- ms$clustering
           names(cl$clustering) <- names(x[,1])
           cl$certainty <- ms$certainty
           names(cl$certainty) <- names(x[,1])
           v <- ms$score
         })

  r <- list(k=clNum, score=v, clustering=cl$clustering, sizes=sizes,
            certainty=cl$certainty)
  return(r)
}

.mclustScore <- function(x, k=2, subset=NULL) {
  nobs <- nrow(x)

  if(!is.null(subset))
    x <- x[-subset,]
  tmp1 <- try(suppressWarnings(uni <- summary(EMclust(x, 1, c("EII", "VII")), x)))
  tmp2 <- try(suppressWarnings(bi <- summary(EMclust(x, k, c("EII", "VII")), x)))
  if(class(tmp1) == "try-error" | class(tmp2) == "try-error") {
    return(list(score=0,
                clustering=0, certainty=0))
  }
  scores <- (bi$loglik[1] - uni$loglik[1]) / nobs
  list(score=scores,
       clustering=bi$classification, certainty=(1 - bi$uncertainty))
}


.silScore <- function(cl) {
  v <- cl$clusinfo[,1] / sum(cl$clusinfo[,1])
  return(cl$sil$avg.width *
         (-sum(v * log2(v))))
}

setMethod("clusterGeneClass", "pcaRes",
          function(object, clNum=2, scoretype=c("mclust", "weightedsilhouette", "silhouette"),...) 
            .clusterGeneClass(object@scores, clNum=clNum, scoretype=scoretype))

setMethod("clusterGeneClass", "matrix", 
          function(object, clNum=2, scoretype=c("mclust", "weightedsilhouette", "silhouette"),...) 
            .clusterGeneClass(object, clNum=clNum, scoretype=scoretype))




