\name{GOhelpers}
\alias{getFlavorSubmap}
\alias{isFlavor}
\alias{getGOTerm}

\title{Small convenience functions for dealing with GO}
\description{\code{getFlavorSubmap} extracts a submap of a large GO
  map. Selects all items whose names are GO identifiers that belong to
  the <flavor> part of the GO hierarchy. \code{isFlavor} checks if a GO
  class is of requested flavor and \code{getGOTerm} extracts the term
  corresponding to a given GO identifier.}
\usage{getFlavorSubmap(map, flavor=c("CC", "MF", "BP"))
isFlavor(goclass, flavor)
getGOTerm(goclass)
}
\arguments{
  \item{map}{Parent mapping, typically obtained from the GO package.}
  \item{flavor}{One of CC, MF or BP for cellular component, molecular
    function or biological process}
  \item{goclass}{A GO identifier (or vector thereof), eg "GO:0006412"}
}
\value{
  \item{getFlavorSubmap}{Returns the submap, a list where each components is a character vector
    containing the identifers of the members in that class}
  \item{isFlavor}{logical}
  \item{getGOTerm}{Character vector with corresponding terms}
}
\author{Henning Redestig}
\seealso{\code{GO}}
\examples{
require("GO") || stop("GO unavailable")
# Convert a GO environment object to a list
map <- as.list(GOLOCUSID)
submap <- getFlavorSubmap(map, "BP")
isFlavor("GO:0006412", "BP")
getGOTerm("GO:0006412")
}
\keyword{list}
