\name{capiu}
\alias{capiu}
\title{Clustering using A Priori Information via Unsupervised decision trees.}
\description{A method for generating unsupervised decision trees for
  e.g. gene expression or similar data that provide variables which can
  be grouped in \emph{classes} for separate analysis.}
\usage{capiu(x, map, labels=NULL,...)}
\arguments{
  \item{x}{A numerical matrix or exprSet with rows as variables and samples as
    columns.}
  \item{map}{A list that specifies which class each variable belongs
    to.}
  \item{labels}{Optional labels for the samples to be used in
    visualizations.}
  \item{...}{Further arguments to \code{batchPca}, \code{scoreModels},
    \code{nodeGen}}
}

\details{Consider a gene expression data matrix with expected subgroups
  in the samples and a mapping that defines classes of genes that alone
  can be assumed to provide more information about some biological
  process (or similar) than all genes do together. Then PCA models can
  be built for each of the gene classes alone and these models can then
  be examined to see if they provide a 'natural' partitioning of the
  samples such as 'treated' versus 'non-treated', the label classes that
  do so can then be hypothesised to be related to the biological reasons
  for the groupings. By then splitting the data using only this class
  and the re-partition the data in each node, the subgroups of samples
  can be (hopefully) be recovered in the leafs of the obtained tree.

  \code{capiu} is a wrap-up function that combines \code{batchPca},
  \code{scoreModels} and \code{nodeGen} to one interface for computing
  unsupervised decision trees. The result can be visualised using dot and
  \code{toDot}.

  Unfortunately, \code{capiu} is a very slow function as it is 100\% R
  and relies on a slow bootstrapping test. Setting \code{K} to a low
  value, \code{correction} to "none" and increasing
  \code{minClustersize} obviously makes it quicker
  but then no real checking for significance is done.

  The extra parameters to give as \code{...} are typically:
  \describe{
    \item{For \code{batchPca}:}{If the PCA preprocessing step of each
      gene class should scale the data or not by specifying
      \code{scale}. How many PC's that should be extracted by eg
      specifying \code{varLimit}.}
    \item{For \code{scoreModels}:}{How the gene classes should be scored by
      specifying \code{scoretype}}
    \item{For \code{nodeGen}:}{How small the smallest cluster is expected
      to be be specifying \code{minClusterSize}, how similar to
      clusterings must be to be considered the same by specifying
      \code{minSimilarity}, how many bootstrap replicates should be
      calculated for the the significance test by specifying \code{K}}
    }
}
\value{
  A udt object.
}
\author{Henning Redestig}
\seealso{\code{nodeGen}, \code{batchPca}, \code{scoreModels}}
\examples{
data(golub)
golubMap  <-  makeMap("hu6800", geneNames(golub), upperLimit=200)
#Warning, this takes a long time to execute!
\dontrun{
golubTree  <-  capiu(golub,
                     golubMap,
                     varLimit=0.3,
                     K=2000,
                     labels=paste(golub[["ALL.AML"]], golub[["T.B.cell"]]))
}
#You need dot installed to compile script generated by toDot
\dontrun{
toDot(golubTree, "golubTree.dot")
}
}
\keyword{tree}
