\name{Lazy Model Bootstrap}
\alias{lazyModelBootstrap}
\title{Perform a simplified version of the bootstrap test for modelScores}
\description{Can be used to test how much better a score for a gene
  class is than one would expect if the data came from a zero mean
  multivariate normal distribution with Sigma equal to that of the gene
  class (or the by PCA summarized gene class)}
\usage{lazyModelBootstrap(models, scores, K=2000, null=1, alt=2,
correction="BY", verbose=TRUE, ...)}
\arguments{
  \item{models}{A batchPcaRes object}
  \item{scores}{A modelScores object}
  \item{K}{The amount of replica to consider}
  \item{null}{The modality of the null distribution}
  \item{alt}{The modalitive of the alternative hypothesis}
  \item{correction}{The P-value correction to use see \code{p.adjust}}
  \item{verbose}{Print information about progress}
  \item{...}{Passes through arguments}
}
\details{
  This method computes a set of samples, for each cardinality of the
  batchPcaRes object, from a multivariate distribution
  under the null-hypothesis with mean zero and Sigma sampled from the
  set of Sigmas in the batchPcaRes object. So if the elements of
  batchPcaRes has either 1 or 2 PCs, 2 * K replicates will be
  computed. These are then scored the same way the models were scored
  and a p-value is calculated as the amount of samples greater than the
  observed under null hypothesis divided by K.
}

\value{
  A list of corrected and uncorrected p-values.
}
\author{Henning Redestig}
\examples{
data(golubMergeSub)
myRandomMapping <- sapply(1:10, function(x) sample(1:1000, sample(5:200)))
models <- batchPca(golubMergeSub, myRandomMapping)
scores <- scoreModels(models)
pvals <- lazyModelBootstrap(models, scores, K=100)
}
\keyword{multivariate}
